@@ -12,7 +12,7 @@ end
 
 const cv = OpenCV
 net = cv.dnn.DetectionModel(joinpath(ENV["OPENCV_TEST_DATA_PATH"], "dnn", "opencv_face_detector.pbtxt"),joinpath(ENV["OPENCV_TEST_DATA_PATH"], "dnn", "opencv_face_detector_uint8.pb"))
-size0 = Int32(300)
+size0 = 300
 
 cv.dnn.setPreferableTarget(net, cv.dnn.DNN_TARGET_CPU)
 cv.dnn.setInputMean(net, (104, 177, 123))
@@ -22,7 +22,7 @@ cv.dnn.setInputSize(net, size0, size0)
 
 img = OpenCV.imread(joinpath(test_dir, "cascadeandhog", "images", "mona-lisa.png"))
 
-classIds, confidences, boxes = cv.dnn.detect(net, img, confThreshold=Float32(0.5))
+classIds, confidences, boxes = cv.dnn.detect(net, img, confThreshold=0.5)
 
 box = (boxes[1].x, boxes[1].y, boxes[1].x+boxes[1].width, boxes[1].y+boxes[1].height)
 expected_rect = (185,101,129+185,169+101)
